--[[
Name: Console-1.0
Revision: $Rev: 41 $
Author(s): sylvanaar (sylvanaar@mindspring.com)
Original Author: Norganna (see below)
Documentation: http://www.wowace.com/wiki/Debug-1.0
SVN: http://svn.wowace.com/root/trunk/Debug-1.0/Console-1.0
Description: A log viewer console
Dependencies: AceLibrary, Ace-00
License: LGPL v2.1
]]


--[[
    ORGINAL ADDON HEADER:

	nLog - A debugging console for World of Warcraft.
	$Id: LogConsole-1.0.lua 41 2008-10-09 21:55:12Z nevcairiel $
	Copyright (C) 2006 Norganna

	This library is free software; you calo redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
]]

-- /run for i = 1, 1000 do log.AddMessage("Auctioneer", "Scan",  1, "Empty Auction "..i, "Found empty auction with item "..i) end

-- Call log.AddMessage(addon, title, type, level, detail)
-- Eg: /script log.AddMessage("LogConsole", "Test", LOG_CRITICAL, "", "An Important Message")title

local util, _, CLR = GetUtils()

local _G = getfenv(0)

local function setupenv(env, fromenv)
    -- Message Levels
    env.LOG_CRITICAL = 1
    env.LOG_WARNING = 2
    env.LOG_NOTICE = 3
    env.LOG_INFO = 4
    env.LOG_DEBUG = 5
    
    if fromenv and fromenv.log then
        env.log = fromenv.log
    else 
        env.log = {
        	messages = {},
        	levels = {
        		[LOG_CRITICAL] = "Severe",
        		[LOG_WARNING] = "Warn",
        		[LOG_NOTICE] = "Note",
        		[LOG_INFO] = "Info",
        		[LOG_DEBUG] = "Debug",
        	}
        }
    end
end

setupenv(_G)

local nLog = _G.log
local VIEWER_FONT = "Interface\\AddOns\\Debug-1.0\\VeraMono.TTF"



nLog.Version= 90000 + tonumber(("$Revision: 41 $"):match("(%d+)"))

NLOG_VERSION = nLog.Version

nLogData = {
	enabled = true,
	
}

nLog.autoScroll = true


function nLog.ChatMsg(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg)
end

local chat = nLog.ChatMsg
local broken = false
if not p then p = function () end end

function nLog.IsEnabled()
	return nLogData.enabled
end


function dumpex(frame, name, order, a1, ...)
    local s = ""
    local order_tmp
    
    if order then
        order_tmp = util:acquire()
        for k,v in pairs(order) do order_tmp[k] = v end
    end

    if select('#', ...) == 0 then
        s = util:literal_tostring(a1, true, order_tmp and util:merge_keys(order_tmp, a1))
    else
        s = (", "):join(util:literal_tostring_args_ordered(order_tmp, a1, ...))
    end
    
    if order_tmp then 
        util:reclaim(order_tmp)
    end
    
    return s or ""
end


function dump(a1, ...)
    if type(a1) == "table" and type(rawget(a1, 0)) == "userdata" and type(a1.GetObjectType) == "function" then 
        return dumpex(a1, ...)
    else
        return dumpex(nil, nil, nil, a1, ...)
    end
end

local function format(...)
	local n = select("#", ...)
	local out = ""
	for i = 1, n do
		if i > 1 and out:sub(-1) ~= "  " then out = out .. " "; end
		local d,d2,d3,d4,d5 = select(i, ...)
		if (type(d) == "string") then
			if (d:sub(1,1) == " ") then
				out = out .. d:sub(2)
			else
				out = out..d;
			end
		else
		    if d == nil and d2 == nil and d3 == nil and d4 ~= nil then
			    out = out..dumpex(d,d2,d3,d4)
		        i = i + 3
		    else
		        if type(d) == "table" and type(rawget(d, 0)) == "userdata" and type(d.GetObjectType) == "function" then 
		            out = out..dumpex(d,d2,d3,d4)
		            i=i+3
		        else
		            out = out..dump(d)
		        end
		    end
		end
	end
	return out
end


-- local dbginfo = { addon = "", component = "", severity = 0, label = "" }
DEBUGLIB_DEBUG_FRAME = nLog
function nLog.Msg(info, ...)
    nLog.AddMessage(info.addon or "", 
        info.component or "", 
        info.severity and (info.severity > 0 and info.severity) or LOG_DEBUG, 
        info.label or "",
         ...)
end

local function scrollToBottom(force)
   if nLog.autoScroll or force then 
       local min, max = Debug10LogMessageScrollScrollBar:GetMinMaxValues()
       Debug10LogMessageScrollScrollBar:SetValue(max)
   end
end

local pid = 0
function nLog.AddMessage(mAddon, mType, mLevel, mTitle, ...)
	if broken then return end
	if not (nLogData.enabled and mAddon and mType and mLevel and mTitle) then return end
	local ts = date("%b %d %H:%M:%S");
	pid = pid + 1
	local msg = format(...)

	-- Once we fill up to 64k entries, treat the table like a loop buffer
	if (#nLog.messages >= 65535) then
		local ofs = nLog.ofs or 0
		local message = nLog.messages[ofs+1]
		if (not message) then chat("Error logging message at offset("..ofs..")") broken = true return end
		message[1] = ts
		message[2] = pid
		message[3] = mAddon
		message[4] = mType
		message[5] = mLevel
		message[6] = mTitle
		message[7] = msg
		nLog.ofs = (ofs+1 % 65535)
		return
	end
	table.insert(nLog.messages, {
		ts, pid, mAddon, mType, mLevel, mTitle, msg
	})

end






local updateInterval = 1.0
local updated = 0
function nLog.OnUpdate(frame, delay)
    if frame:IsShown() then 
    	updated = updated + delay

        while (updated > updateInterval) do
            nLog.UpdateDisplay()
            updated = updated - updateInterval;
        end
    end
end

function nLog.MessageShow()
    if     Debug10LogMessageScrollScrollBarScrollDownButtonFlash == nil then 
        nLog.AddFlashTexture(Debug10LogMessageScrollScrollBarScrollDownButton)
    end



	nLog.Message.pos = table.getn(nLog.messages)
	nLog.MessageDisplay()
end

function nLog.MessageDisplay(id)
	nLog.MessageUpdate()
	nLog.Message:Show()
	
    if nLog.autoScroll then
        scrollToBottom()
    end
end

function nLog.MessageDone()
	nLog.Message:Hide()
end

function nLog.MessageClicked(...)
end



function nLog.MessageUpdate()
	nLog.Message.BoxScroll:UpdateScrollChildRect()
end

nLog.filtered = {}
function nLog.FilterUpdate()
	nLog.filterLevel = nLog.Message.LevelFilt:GetValue()
	nLog.filterAddon = nLog.Message.AddonFilt:GetText()
	nLog.filterType = nLog.Message.TypeFilt:GetText()
	nLog.filterLabel = nLog.Message.LabelFilt:GetText()

	if nLog.filtered.filterLevel == nLog.filterLevel
	and nLog.filtered.filterAddon == nLog.filterAddon
	and nLog.filtered.filterType == nLog.filterType
	and nLog.filtered.filterLabel == nLog.filterLabel
	and nLog.filtered.count == #nLog.messages
	and nLog.filtered.ofs == nLog.ofs
	then
		return
	end

	-- Clean out the filter list and update
	for key in ipairs(nLog.filtered) do
		nLog.filtered[key] = nil
	end

	-- Rebuild the filter list
	local message, mAddon, mType, mLevel, mLabel
	nLog.filtered.filterLevel = nLog.filterLevel
	nLog.filtered.filterAddon = nLog.filterAddon
	nLog.filtered.filterType = nLog.filterType
	nLog.filtered.filterLabel = nLog.filterLabel
	nLog.filtered.count = #nLog.messages
	nLog.filtered.ofs = nLog.ofs

	local fLevel = tonumber(nLog.filtered.filterLevel)
	local fAddon = (nLog.filtered.filterAddon or ""):lower()
	local fType = (nLog.filtered.filterType or ""):lower()
	local fLabel = (nLog.filtered.filterLabel or ""):lower()
	local fOfs = nLog.filtered.ofs or 0
	if (fAddon == "") then fAddon = nil end
	if (fType == "") then fType = nil end
	if (fLabel == "") then fLabel = nil end
	for i = 0, #nLog.messages-1 do
		local fIdx = ((i + fOfs) % 65535)+1
		message = nLog.messages[fIdx]
		mAddon, mType, mLevel, mLabel = message[3], message[4], message[5], message[6]
		if (not fLevel or fLevel >= mLevel)
		and (not fAddon or fAddon == mAddon:sub(1, #fAddon):lower())
		and (not fType or fType == mType:sub(1, #fType):lower())
		and (not fLabel or fLabel == mLabel:sub(1, #fLabel):lower())
		then
			table.insert(nLog.filtered, fIdx)
		end
	end
	
end

local clrcache = {}
local function GetAutoGenColorCLR(Name)
    if Name and clrcache[Name] then 
        return clrcache[Name]
    end
    
    if not Name then return "ffffff" end
        
	local hash = 17
	for i=1, Name:len() do
		hash = hash * 37 * Name:byte(i);
	end

	local r = math.floor(math.fmod(hash / 97, 255));
	local g = math.floor(math.fmod(hash / 17, 255));
	local b = math.floor(math.fmod(hash / 227, 255));

    if ((r * 299 + g * 587 + b * 114) / 1000) < 105 then
    	r = math.abs(r - 255);
        g = math.abs(g - 255);
        b = math.abs(b - 255);
    end

    clrcache[Name] = ("%02x%02x%02x"):format(r, g, b)
	return clrcache[Name]
end

--
-- Decorator Pattern Text Colorization Functions
-- Same as crayonlib
--
local CLR = {}
CLR.COLOR_NONE = nil
function CLR:Auto(txt) return CLR:Colorize(GetAutoGenColorCLR(txt), txt) end
function CLR:AutoKey(txt, key) return CLR:Colorize(GetAutoGenColorCLR(key), txt) end

function CLR:Colorize(hexColor, text)
    if text == nil then text = "" end

    if hexColor == CLR.COLOR_NONE then
        return text
    end

    return "|cff" .. tostring(hexColor or 'ffffff') .. tostring(text) .. "|r"
end
function CLR:GetHexColor(color)
    return string.format("%02x%02x%02x", color.r*255, color.g*255, color.b*255)
end

local function isAtBottom()
   local min, max = Debug10LogMessageScrollScrollBar:GetMinMaxValues()
   return  Debug10LogMessageScrollScrollBar:GetValue() >= max
end


function nLog.CheckScrollWarn()
    local f = Debug10LogMessageScrollScrollBarScrollDownButtonFlash
    if not f then return end
    local bottom = isAtBottom()
    

    if bottom and f:IsShown() then 
        f:Hide()
        UIFrameFlashRemoveFrame(f)
    elseif not bottom and not f:IsShown() then 
        f:Show()
        UIFrameFlash(f, 0, 0, -1, false, 1,1)            
    end
end


local function wrn(msg)
    nLog.AddMessage("LogConsole", "Test", LOG_WARNING, "", msg)
end

function nLog.LineClicked(frame)
    nLog.autoScroll = false
    Debug10LogMessageScrollScrollBarScrollDownButtonFlash:Hide()
    wrn("AutoScroll OFF")
	local idx = frame.idx
	local ts, mId, mAddon, mType, mLevel, mTitle, msg = unpack(nLog.messages[idx])
	local mLevelName = nLog.levels[mLevel]
	local text = string.format("|cffffaa11Date:|r  %s\n|cffffaa11MsgId:|r %s\n|cffffaa11AddOn:|r %s\n|cffffaa11Title:|r  %s\n|cffffaa11Severity:|r %s\n|cffffaa11Subsystem:|r %s\n|cffffaa11Message:|r\n%s\n", 
	        ts, mId, CLR:Auto(mAddon), CLR:Auto(mTitle), CLR:Auto(mLevelName), CLR:Auto(mType),  msg)	   
	nLog.Message.Box:SetText(text)
end


local lastLine = 0
local LOG_LINES = 16
function nLog.UpdateDisplay()
	local message, ts, mId, mAddon, mType, mLevel, mLevelName, mTitle, msg, idx, midx
	nLog.FilterUpdate()

	local rows = #nLog.filtered
	local scrollrows = rows
	if (scrollrows > 0 and scrollrows < LOG_LINES+1) then scrollrows = LOG_LINES+1 end
    

	FauxScrollFrame_Update(Debug10LogMessageScroll, scrollrows, LOG_LINES, 16)
    
  	if nLog.autoScroll then
        scrollToBottom()
    end
    
      
	local cpos = FauxScrollFrame_GetOffset(Debug10LogMessageScroll)
	if (cpos ~= lastline) then
		lastline = cpos
	end

	for i = 1, LOG_LINES do
		idx = cpos + i
		midx = nLog.filtered[idx]
		local m = nLog.Message.Lines[i]
		if (midx) then
			message = nLog.messages[midx]	
			if (message) then
				ts, mId, mAddon, mType, mLevel, mTitle, msg = unpack(message)
				mLevelName = nLog.levels[mLevel]
				
				m:SetText(string.format("%s-%s%s %s: %.80s", 
				    CLR:AutoKey(("%-6.6s"):format(mLevelName), mLevelName), CLR:Auto(mAddon), 
				    (mTitle ~= "") and ("["..CLR:Auto(mTitle).."]") or "", 
				    (mType ~= "") and (" <"..CLR:Auto(mType)..">") or "",   msg))
				m:Show()
				m.idx = midx
			else
				m:Hide()
			end
		else
			m:Hide()
		end
	end
end

function nLog.ClearLog()
	nLog.messages = {}
	nLog.UpdateDisplay()
	nLog.Message.Box:SetText("")
	chat("Clearing messages.")
end

local function showTooltip(obj)
	local tooltip = obj.tooltip
	GameTooltip:SetOwner(obj, "ANCHOR_RIGHT")
	GameTooltip:ClearLines()
	GameTooltip:AddLine(tooltip)
	GameTooltip:Show()
end

local function hideTooltip()
	GameTooltip:Hide()
end




local function createFrame()
    
    -- Create our message message frame
    nLog.Message = CreateFrame("Frame", "Debug10LogFrame", UIParent)
    nLog.Message:Hide()
    nLog.Message:SetPoint("CENTER", "UIParent", "CENTER")
    nLog.Message:SetFrameStrata("DIALOG")
    nLog.Message:SetHeight(400)
    nLog.Message:SetWidth(600)
    nLog.Message:SetBackdrop({
    	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
    	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
    	tile = true, tileSize = 32, edgeSize = 16,
    	insets = { left = 4, right = 4, top = 4, bottom = 4 }
    })
    nLog.Message:SetBackdropColor(0,0,0.5, 0.8)
    nLog.Message:SetScript("OnShow", nLog.MessageShow)
    nLog.Message:SetMovable(true)
    
    nLog.Message.Drag = CreateFrame("Button", "", nLog.Message)
    nLog.Message.Drag:SetPoint("TOPLEFT", nLog.Message, "TOPLEFT", 10,-5)
    nLog.Message.Drag:SetPoint("TOPRIGHT", nLog.Message, "TOPRIGHT", -10,-5)
    nLog.Message.Drag:SetHeight(6)
    nLog.Message.Drag:SetHighlightTexture("Interface\\FriendsFrame\\UI-FriendsFrame-HighlightBar")
    nLog.Message.Drag:SetScript("OnMouseDown", function() nLog.Message:StartMoving() end)
    nLog.Message.Drag:SetScript("OnMouseUp", function() nLog.Message:StopMovingOrSizing() end)
    
    nLog.Message.AddonFilt = CreateFrame("EditBox", "Debug10LogAddFilt", nLog.Message, "InputBoxTemplate")
    nLog.Message.AddonFilt:SetPoint("BOTTOMLEFT", nLog.Message, "BOTTOMLEFT", 20, 12)
    nLog.Message.AddonFilt:SetAutoFocus(false)
    nLog.Message.AddonFilt:SetHeight(15)
    nLog.Message.AddonFilt:SetWidth(80)
    nLog.Message.AddonFilt.tooltip = "Filter by AddOn"
    nLog.Message.AddonFilt:SetScript("OnEnter", showTooltip)
    nLog.Message.AddonFilt:SetScript("OnLeave", hideTooltip)

    nLog.Message.LabelFilt = CreateFrame("EditBox", "Debug10LogLabelFilt", nLog.Message, "InputBoxTemplate")
    nLog.Message.LabelFilt:SetPoint("BOTTOMLEFT", nLog.Message.AddonFilt, "BOTTOMRIGHT", 5, 0)
    nLog.Message.LabelFilt:SetAutoFocus(false)
    nLog.Message.LabelFilt:SetHeight(15)
    nLog.Message.LabelFilt:SetWidth(80)
    nLog.Message.LabelFilt.tooltip = "Filter by Title"
    nLog.Message.LabelFilt:SetScript("OnEnter", showTooltip)
    nLog.Message.LabelFilt:SetScript("OnLeave", hideTooltip)
    
    nLog.Message.TypeFilt = CreateFrame("EditBox", "Debug10LogTypeFilt", nLog.Message, "InputBoxTemplate")
    nLog.Message.TypeFilt:SetPoint("BOTTOMLEFT", nLog.Message.LabelFilt, "BOTTOMRIGHT", 5, 0)
    nLog.Message.TypeFilt:SetAutoFocus(false)
    nLog.Message.TypeFilt:SetHeight(15)
    nLog.Message.TypeFilt:SetWidth(80)
    nLog.Message.TypeFilt.tooltip = "Filter by Subsytem"
    nLog.Message.TypeFilt:SetScript("OnEnter", showTooltip)
    nLog.Message.TypeFilt:SetScript("OnLeave", hideTooltip)
    

    nLog.Message.LevelFilt = CreateFrame("Slider", "Debug10LevelFilt", nLog.Message, "OptionsSliderTemplate")
    nLog.Message.LevelFilt:SetPoint("LEFT", nLog.Message.TypeFilt, "RIGHT", 60, 0)
    nLog.Message.LevelFilt:SetHeight(20)
    nLog.Message.LevelFilt:SetWidth(60)

    Debug10LevelFiltLow:SetText(nLog.levels[1])
    Debug10LevelFiltHigh:SetText(nLog.levels[5])
    

    Debug10LevelFiltLow:ClearAllPoints()
    Debug10LevelFiltHigh:ClearAllPoints()

    Debug10LevelFiltLow:SetPoint("RIGHT", Debug10LevelFilt, "LEFT", -5,0)
    Debug10LevelFiltHigh:SetPoint("LEFT", Debug10LevelFilt, "RIGHT", 5, 0)



    nLog.Message.LevelFilt:SetMinMaxValues(1, 5)
    nLog.Message.LevelFilt:SetValueStep(1)
    nLog.Message.LevelFilt:SetValue(LOG_DEBUG)
    nLog.Message.LevelFilt:SetHitRectInsets(0,0,0,0)
    nLog.Message.LevelFilt.tooltip = "Filter by maximum message severity type (1=critical, 2=warning, 3=notice, 4=info, 5=debug)"
    nLog.Message.LevelFilt:SetScript("OnEnter", showTooltip)
    nLog.Message.LevelFilt:SetScript("OnLeave", hideTooltip)
    
    nLog.Message.Done = CreateFrame("Button", "", nLog.Message, "OptionsButtonTemplate")
    nLog.Message.Done:SetText("Close")
    nLog.Message.Done:SetPoint("BOTTOMRIGHT", nLog.Message, "BOTTOMRIGHT", -10, 10)
    nLog.Message.Done:SetScript("OnClick", nLog.MessageDone)
    
    nLog.Message.MsgScroll = CreateFrame("ScrollFrame", "Debug10LogMessageScroll", nLog.Message, "FauxScrollFrameTemplate")
    nLog.Message.MsgScroll:SetPoint("TOPLEFT", nLog.Message, "TOPLEFT", 20, -20)
    nLog.Message.MsgScroll:SetPoint("RIGHT", nLog.Message, "RIGHT", -40, 0)
    nLog.Message.MsgScroll:SetHeight(200)


    nLog.Message.MsgScroll:SetScript("OnVerticalScroll", function () 
        FauxScrollFrame_OnVerticalScroll(LOG_LINES, nLog.UpdateDisplay) 
            
            if not nLog.autoScroll and isAtBottom() then 
                nLog.autoScroll = true 
                wrn("AutoScroll ON")
            elseif nLog.autoScroll then
                scrollToBottom()
            end
            
            nLog.CheckScrollWarn()
        end)
    nLog.Message.MsgScroll:SetScript("OnShow", function() nLog.UpdateDisplay() end)
    
    nLog.Message.BoxFrame = CreateFrame("Frame", "", nLog.Message)
    nLog.Message.BoxFrame:SetBackdrop({
    	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
    	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
    	tile = true, tileSize = 32, edgeSize = 16,
    	insets = { left = 4, right = 4, top = 4, bottom = 4 }
    })
    nLog.Message.BoxFrame:SetBackdropColor(0,0,0.5, 0.8)
    nLog.Message.BoxFrame:SetPoint("TOPLEFT", nLog.Message.MsgScroll, "BOTTOMLEFT", -5, 0)
    nLog.Message.BoxFrame:SetPoint("RIGHT", nLog.Message, "RIGHT", -15, 0)
    nLog.Message.BoxFrame:SetPoint("BOTTOM", nLog.Message.Done, "TOP", 0, 5)
    
    nLog.Message.BoxScroll = CreateFrame("ScrollFrame", "Debug10LogMessageInputScroll", nLog.Message.BoxFrame, "UIPanelScrollFrameTemplate")
    nLog.Message.BoxScroll:SetPoint("TOPLEFT", nLog.Message.BoxFrame, "TOPLEFT", 10, -5)
    nLog.Message.BoxScroll:SetPoint("BOTTOMRIGHT", nLog.Message.BoxFrame, "BOTTOMRIGHT", -27, 4)
    
    nLog.Message.Box = CreateFrame("EditBox", "Debug10LogMessageEditBox", nLog.Message.BoxScroll)
    nLog.Message.Box:SetFont(VIEWER_FONT, 11)
    nLog.Message.Box:SetPoint("BOTTOM", nLog.Message.BoxFrame, "BOTTOM", 0,0)
    nLog.Message.Box:SetWidth(550)
    nLog.Message.Box:SetMultiLine(true)
    nLog.Message.Box:SetAutoFocus(false)
    nLog.Message.Box:SetFontObject(GameFontHighlight)
    nLog.Message.Box:SetScript("OnEscapePressed", nLog.MessageDone)
    nLog.Message.Box:SetScript("OnTextChanged", nLog.MessageUpdate)
    nLog.Message.Box:SetScript("OnEditFocusGained", nLog.MessageClicked)
    nLog.Message.Box:SetText("|cffffa011Select a message above to view it's contents.|r")
    nLog.Message.BoxScroll:SetScrollChild(nLog.Message.Box)
    
    nLog.Message.Lines = {}
    for i=1, 16 do
    	local line = CreateFrame("Button", "", nLog.Message)
    	nLog.Message.Lines[i] = line
    	line.id = i
    	if (i == 1) then
    		line:SetPoint("TOPLEFT", nLog.Message.MsgScroll, "TOPLEFT", 0, 0)
    	else
    		line:SetPoint("TOPLEFT", nLog.Message.Lines[i-1], "BOTTOMLEFT", 0, 0)
    	end
    	line:SetPoint("RIGHT", nLog.Message.MsgScroll, "RIGHT", -5, 0)
    	line:SetHeight(12)
    	line:SetScript("OnClick", nLog.LineClicked)
    	line.text = line:CreateFontString("", "HIGH")
    	line.text:SetPoint("LEFT", line, "LEFT")
    	line.text:SetPoint("RIGHT", line, "RIGHT")
    	line.text:SetFont(VIEWER_FONT, 11)
    	line.text:SetJustifyH("LEFT")
    	line.SetText = function(obj, ...) obj.text:SetText(...) end
    	line:SetText("LINE "..i)
    end


   nLog.Frame:SetScript("OnUpdate", nLog.OnUpdate) 
end

-- /script UIFrameFlash(Debug10LogMessageScrollScrollBarDownButtonFlash, 0, 0, -1, false, 1,1)
function nLog.AddFlashTexture(frame)
    -- define variables used
    local t = frame:CreateTexture(frame:GetName().."Flash", "OVERLAY")
    -- define the parameters for the texture
    t:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-BlinkHilight")
    t:SetPoint("CENTER", frame, "CENTER", 0, 1)
    t:SetBlendMode("ADD")
    t:SetAlpha(0.5)
    -- hide the texture by default
    t:Hide()
end    

function nLog.OnEvent(frame, event, ...)
	if (event == "PLAYER_ENTERING_WORLD") then	   
		frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
		createFrame()

        local sml = AceLibrary:HasInstance("SharedMedia-1.0", false) and AceLibrary("SharedMedia-1.0");
        if sml then
            sml:Register(sml.MediaType.FONT, "VeraMono", VIEWER_FONT)
        end
		--nLog.Message:Show()
	end
end


nLog.Frame = CreateFrame("Frame")
nLog.Frame:SetScript("OnEvent", nLog.OnEvent)
nLog.Frame:RegisterEvent("PLAYER_ENTERING_WORLD")




_G.SLASH_LOG1 = "/log"
_G.SLASH_LOG2 = "/logcon"
_G.SlashCmdList["LOG"] = function(msg)
	if (not msg or msg == "" or msg == "help") then
		chat("Log help:")
		chat("  /log enable    -  Enables log")
		chat("  /log disable   -  Disables log")
		chat("  /log show      -  Shows the log frame")
		chat("  /log clear     -  Clears current log messages")
	elseif (msg == "show") then
        nLog.Message:Show()
	elseif (msg == "enable") then
		nLogData.enabled = true
		chat("Log will now catch messages")
	elseif (msg == "disable") then
		nLogData.enabled = false
		chat("Log will no longer catch messages")
	elseif (msg == "clear") then
		nLog.ClearLog()
	else
		chat("Unknown command: "..(msg or "nil"))
	end
end

